from __future__ import annotations
import logging
from pathlib import Path
import subprocess
import sys
import asyncclick as click
from asyncclick.testing import CliRunner
import pytest
from click_loglevel import LogLevel

DATA_DIR = Path(__file__).with_name("data")


@click.command()
@click.option("-l", "--log-level", type=LogLevel(), default=logging.INFO)
async def lvlcmd(log_level: int) -> None:
    click.echo(repr(log_level))


@click.command()
@click.option("-l", "--log-level", type=LogLevel())
async def lvlcmd_nodefault(log_level: int) -> None:
    click.echo(repr(log_level))


STANDARD_LEVELS = [
    ("CRITICAL", logging.CRITICAL),
    ("critical", logging.CRITICAL),
    ("cRiTiCaL", logging.CRITICAL),
    (logging.CRITICAL, logging.CRITICAL),
    ("ERROR", logging.ERROR),
    ("error", logging.ERROR),
    ("ErRoR", logging.ERROR),
    (logging.ERROR, logging.ERROR),
    ("WARNING", logging.WARNING),
    ("warning", logging.WARNING),
    ("WaRnInG", logging.WARNING),
    (logging.WARNING, logging.WARNING),
    ("INFO", logging.INFO),
    ("info", logging.INFO),
    ("iNfO", logging.INFO),
    (logging.INFO, logging.INFO),
    ("DEBUG", logging.DEBUG),
    ("debug", logging.DEBUG),
    ("dEbUg", logging.DEBUG),
    (logging.DEBUG, logging.DEBUG),
    ("NOTSET", logging.NOTSET),
    ("notset", logging.NOTSET),
    ("NoTsEt", logging.NOTSET),
    (logging.NOTSET, logging.NOTSET),
    (42, 42),
    (" 42 ", 42),
]


@pytest.mark.anyio
@pytest.mark.parametrize("loglevel,value", STANDARD_LEVELS)
async def test_loglevel(loglevel: str | int, value: int) -> None:
    r = await CliRunner().invoke(lvlcmd, ["-l", str(loglevel)])
    assert r.exit_code == 0, r.output
    assert r.output == str(value) + "\n"


@pytest.mark.anyio
async def test_loglevel_default() -> None:
    r = await CliRunner().invoke(lvlcmd)
    assert r.exit_code == 0, r.output
    assert r.output == str(logging.INFO) + "\n"


@pytest.mark.anyio
async def test_loglevel_no_default() -> None:
    r = await CliRunner().invoke(lvlcmd_nodefault)
    assert r.exit_code == 0, r.output
    assert r.output == "None\n"


@pytest.mark.anyio
@pytest.mark.xfail(True, reason="asyncclick doesn't honor get_metavar")
async def test_loglevel_help() -> None:
    r = await CliRunner().invoke(lvlcmd, ["--help"])
    assert r.exit_code == 0, r.output
    assert "--log-level [NOTSET|DEBUG|INFO|WARNING|ERROR|CRITICAL]" in r.output


@pytest.mark.anyio
@pytest.mark.parametrize(
    "value",
    [
        "x",
        "logging.INFO",
        "VERBOSE",
    ],
)
async def test_invalid_loglevel(value: str) -> None:
    r = await CliRunner().invoke(lvlcmd, ["--log-level", value])
    assert r.exit_code != 0, r.output
    assert f"{value!r}: invalid log level" in str(r)


@pytest.mark.parametrize(
    "loglevel,value",
    STANDARD_LEVELS
    + [
        ("VERBOSE", 15),
        ("verbose", 15),
        ("VeRbOsE", 15),
        ("NOTICE", 25),
        ("notice", 25),
        ("nOtIcE", 25),
    ],
)
@pytest.mark.parametrize("script", ["list-extra-async.py", "dict-extra-async.py"])
def test_loglevel_extra(loglevel: str | int, value: int, script: str) -> None:
    r = subprocess.run(
        [sys.executable, str(DATA_DIR / script), "--log-level", str(loglevel)],
        stdout=subprocess.PIPE,
        text=True,
    )
    assert r.returncode == 0
    assert r.stdout == str(value) + "\n"


@pytest.mark.xfail(True, reason="asyncclick doesn't honor get_metavar")
@pytest.mark.parametrize("script", ["list-extra-async.py", "dict-extra-async.py"])
def test_loglevel_extra_help(script: str) -> None:
    r = subprocess.run(
        [sys.executable, str(DATA_DIR / script), "--help"],
        stdout=subprocess.PIPE,
        text=True,
    )
    assert r.returncode == 0
    assert (
        "--log-level [NOTSET|DEBUG|INFO|WARNING|ERROR|CRITICAL|VERBOSE|NOTICE]"
        in r.stdout
    )


@pytest.mark.parametrize(
    "value",
    [
        "x",
        "logging.INFO",
        "VERBATIM",
    ],
)
@pytest.mark.parametrize("script", ["list-extra-async.py", "dict-extra-async.py"])
def test_invalid_loglevel_extra(value: str, script: str) -> None:
    r = subprocess.run(
        [sys.executable, str(DATA_DIR / script), "--log-level", value],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    assert r.returncode != 0
    assert f"{value!r}: invalid log level" in r.stderr


@pytest.mark.parametrize(
    "loglevel,value",
    STANDARD_LEVELS
    + [
        ("VERBOSE", 15),
        ("verbose", 15),
        ("VeRbOsE", 15),
        ("NOTICE", 25),
        ("notice", 25),
        ("nOtIcE", 25),
    ],
)
@pytest.mark.parametrize(
    "script",
    [
        "list-extra-nonupper-async.py",
        "dict-extra-nonupper-async.py",
    ],
)
def test_loglevel_extra_nonupper(loglevel: str | int, value: int, script: str) -> None:
    r = subprocess.run(
        [sys.executable, str(DATA_DIR / script), "--log-level", str(loglevel)],
        stdout=subprocess.PIPE,
        text=True,
    )
    assert r.returncode == 0
    assert r.stdout == str(value) + "\n"


@pytest.mark.xfail(True, reason="asyncclick doesn't honor get_metavar")
@pytest.mark.parametrize(
    "script",
    [
        "list-extra-nonupper-async.py",
        "dict-extra-nonupper-async.py",
    ],
)
def test_loglevel_extra_nonupper_help(script: str) -> None:
    r = subprocess.run(
        [sys.executable, str(DATA_DIR / script), "--help"],
        stdout=subprocess.PIPE,
        text=True,
    )
    assert r.returncode == 0
    assert (
        "--log-level [NOTSET|DEBUG|INFO|WARNING|ERROR|CRITICAL|Verbose|Notice]"
        in r.stdout
    )


@pytest.mark.parametrize(
    "value",
    [
        "x",
        "logging.INFO",
        "VERBATIM",
    ],
)
@pytest.mark.parametrize(
    "script",
    [
        "list-extra-nonupper-async.py",
        "dict-extra-nonupper-async.py",
    ],
)
def test_invalid_loglevel_extra_nonupper(value: str, script: str) -> None:
    r = subprocess.run(
        [sys.executable, str(DATA_DIR / script), "--log-level", value],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    assert r.returncode != 0
    assert f"{value!r}: invalid log level" in r.stderr



import logging
import click
from click_loglevel import LogLevel


@click.command()
@click.option(
    "-l",
    "--log-level",
    type=LogLevel(extra={"VERBOSE": 15, "NOTICE": 25}),
)
def main(log_level: int) -> None:
    click.echo(repr(log_level))


if __name__ == "__main__":
    main()



import logging
import click
from click_loglevel import LogLevel

logging.addLevelName(15, "Verbose")
logging.addLevelName(25, "Notice")


@click.command()
@click.option("-l", "--log-level", type=LogLevel(extra=["Verbose", "Notice"]))
def main(log_level: int) -> None:
    click.echo(repr(log_level))


if __name__ == "__main__":
    main()



"""
Log level parameter type for Click

``click-loglevel`` provides a ``LogLevel`` parameter type for use in Click_
programs that wish to let the user set the logging level.  It accepts all of
the ``logging`` log level names (``CRITICAL``, ``ERROR``, ``WARNING``,
``INFO``, ``DEBUG``, and ``NOTSET``, all case insensitive), and converts them
into their corresponding numeric values.  It also accepts integer values and
leaves them as-is.  Custom log levels are also supported.

Starting in version 0.4.0, shell completion of log level names (both built-in
and custom) is also supported.

.. _Click: https://palletsprojects.com/p/click/

Visit <https://github.com/jwodder/click-loglevel> for more information.
"""

from __future__ import annotations
from collections.abc import Iterable, Iterator, Mapping
import logging
import click
from click.shell_completion import CompletionItem

__version__ = "0.6.1"
__author__ = "John Thorvald Wodder II"
__author_email__ = "click-loglevel@varonathe.org"
__license__ = "MIT"
__url__ = "https://github.com/jwodder/click-loglevel"

__all__ = ["LogLevel"]


class LogLevel(click.ParamType):
    """
    A Click parameter type that accepts the standard logging level names (case
    insensitive) and converts them to their corresponding numeric values.  It
    also accepts integer values and leaves them as-is.

    Custom log levels can be added by passing them as the ``extra`` argument to
    the constructor.  ``extra`` can be either an iterable of level names (in
    which case the levels must have already been defined — typically at the
    module level — by calling ``logging.addLevelName()``) or a mapping from
    level names to their corresponding values.  All custom log levels will be
    recognized case insensitively; if two different level names differ only in
    case, the result is undefined.
    """

    name = "log-level"
    LEVELS = ["NOTSET", "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
    __name__ = name

    def __init__(self, extra: Iterable[str] | Mapping[str, int] | None = None) -> None:
        self.levels: dict[str, int] = {lv: getattr(logging, lv) for lv in self.LEVELS}
        level_names = list(self.LEVELS)
        if extra is not None:
            if isinstance(extra, Mapping):
                for lv, value in extra.items():
                    self.levels[lv.upper()] = value
                level_names.extend(extra.keys())
            else:
                for lv in extra:
                    self.levels[lv.upper()] = logging.getLevelName(lv)
                    level_names.append(lv)
        self.metavar = "[" + "|".join(level_names) + "]"

    def convert(
        self, value: str | int, param: click.Parameter | None, ctx: click.Context | None
    ) -> int:
        try:
            return int(value)
        except ValueError:
            assert isinstance(value, str)
            try:
                return self.levels[value.upper()]
            except KeyError:
                self.fail(f"{value!r}: invalid log level", param, ctx)

    def get_metavar(
        self, param: click.Parameter, ctx: click.Context | None = None  # noqa: U100
    ) -> str:
        return self.metavar

    def shell_complete(
        self, _ctx: click.Context, _param: click.Parameter, incomplete: str
    ) -> list[CompletionItem]:
        return [CompletionItem(c) for c in self.get_completions(incomplete)]

    def get_completions(self, incomplete: str) -> Iterator[str]:
        incomplete = incomplete.upper()
        for lv in self.levels:
            if lv.startswith(incomplete):
                yield lv



from __future__ import annotations
import pytest
from click_loglevel import LogLevel


@pytest.mark.parametrize(
    "incomplete,completions",
    [
        ("IN", ["INFO"]),
        ("in", ["INFO"]),
        ("In", ["INFO"]),
        ("info", ["INFO"]),
        ("i", ["INFO"]),
        ("w", ["WARNING"]),
        ("n", ["NOTSET"]),
        ("D", ["DEBUG"]),
        ("E", ["ERROR"]),
        ("c", ["CRITICAL"]),
        ("Q", []),
        ("v", []),
        ("INFOS", []),
    ],
)
def test_get_completions(incomplete: str, completions: list[str]) -> None:
    ll = LogLevel()
    assert list(ll.get_completions(incomplete)) == completions


@pytest.mark.parametrize(
    "incomplete,completions",
    [
        ("IN", ["INFO"]),
        ("in", ["INFO"]),
        ("In", ["INFO"]),
        ("info", ["INFO"]),
        ("i", ["INFO"]),
        ("w", ["WARNING"]),
        ("n", ["NOTSET", "NOTICE"]),
        ("NOT", ["NOTSET", "NOTICE"]),
        ("NOTS", ["NOTSET"]),
        ("NOTI", ["NOTICE"]),
        ("D", ["DEBUG"]),
        ("E", ["ERROR"]),
        ("c", ["CRITICAL"]),
        ("Q", []),
        ("v", ["VERBOSE"]),
        ("INFOS", []),
    ],
)
def test_get_completions_extra(incomplete: str, completions: list[str]) -> None:
    ll = LogLevel(extra={"Verbose": 5, "Notice": 25})
    assert list(ll.get_completions(incomplete)) == completions
